#include <Arduino.h>
#include "USB.h"
#include "USBHIDKeyboard.h"
#include "include/Button.h"

#define ONBOARD_LED_PIN 15 
#define SERIAL_BAUD_RATE 115200
#define LED_COMMON_ANODE true

struct LedConfig {
    uint8_t redPin;
    uint8_t greenPin;
    uint8_t bluePin;
    uint8_t redChannel;
    uint8_t greenChannel;
    uint8_t blueChannel;
};

USBHIDKeyboard Keyboard;

Button buttons[] = {
    Button(1, 'o', 10),  
};

LedConfig leds[] = {
    {2, 4, 6, 0, 1, 2},
};

const int NUM_BUTTONS = sizeof(buttons) / sizeof(Button);
const int NUM_LEDS = sizeof(leds) / sizeof(LedConfig);

void initializeLeds() {
    const int pwmFreq = 5000;
    const int pwmResolution = 8;

    for (int i = 0; i < NUM_LEDS; i++) {

        // set up pins as output
        pinMode(leds[i].redPin, OUTPUT);
        pinMode(leds[i].greenPin, OUTPUT);
        pinMode(leds[i].bluePin, OUTPUT);
        
        // Set up PWM
        ledcSetup(leds[i].redChannel, pwmFreq, pwmResolution);
        ledcSetup(leds[i].greenChannel, pwmFreq, pwmResolution);
        ledcSetup(leds[i].blueChannel, pwmFreq, pwmResolution);
        
        // attach pins to PWM channels
        ledcAttachPin(leds[i].redPin, leds[i].redChannel);
        ledcAttachPin(leds[i].greenPin, leds[i].greenChannel);
        ledcAttachPin(leds[i].bluePin, leds[i].blueChannel);
        
        // Set PWM values to initialize LEDs as off
        if (LED_COMMON_ANODE) {
            ledcWrite(leds[i].redChannel, 255);
            ledcWrite(leds[i].greenChannel, 255);
            ledcWrite(leds[i].blueChannel, 255);
        } else {
            ledcWrite(leds[i].redChannel, 0);
            ledcWrite(leds[i].greenChannel, 0);
            ledcWrite(leds[i].blueChannel, 0);
        }
    }
}

void setLedColor(int ledIndex, uint8_t red, uint8_t green, uint8_t blue) {
    if (ledIndex >= 0 && ledIndex < NUM_LEDS) {
        if (LED_COMMON_ANODE) {
            ledcWrite(leds[ledIndex].redChannel, 255 - red);
            ledcWrite(leds[ledIndex].greenChannel, 255 - green);
            ledcWrite(leds[ledIndex].blueChannel, 255 - blue);
        } else {
            ledcWrite(leds[ledIndex].redChannel, red);
            ledcWrite(leds[ledIndex].greenChannel, green);
            ledcWrite(leds[ledIndex].blueChannel, blue);
        }
    }
}

void turnOffLeds() {
    for (int i = 0; i < NUM_LEDS; i++) {
        setLedColor(i, 0, 0, 0);
    }
}

void flashOnboardLed(int count = 1, int onTime = 100, int offTime = 100) {
    for (int i = 0; i < count; i++) {
        digitalWrite(ONBOARD_LED_PIN, HIGH);
        delay(onTime);
        digitalWrite(ONBOARD_LED_PIN, LOW);
        
        if (i < count - 1) {
            delay(offTime);
        }
    }
}

void updateAndLogButtonStates() {
    for (int i = 0; i < NUM_BUTTONS; i++) {
        bool stateChanged = buttons[i].update();
        
        if (stateChanged) {
            bool currentState = buttons[i].getState();
            if (currentState == LOW) {
                Serial.printf("Button %d PRESSED\n", i);
            } else {
                Serial.printf("Button %d RELEASED\n", i);
            }
        }
    }
}

void setup() {
    Serial.begin(SERIAL_BAUD_RATE);
    
    Keyboard.begin();
    USB.begin();
    
    pinMode(ONBOARD_LED_PIN, OUTPUT);
    digitalWrite(ONBOARD_LED_PIN, LOW);
    
    for (int i = 0; i < NUM_BUTTONS; i++) {
        buttons[i].begin();
    }
    
    initializeLeds();
    
        Serial.println("USB HID Keyboard ready");
        Serial.println("Available commands:");
        Serial.println("  LED <index> <red> <green> <blue> - Set LED color");
        Serial.println("  OFF - Turn all LEDs off");
    
    flashOnboardLed(3, 300, 300);
}

void handleSerialCommand(String command) {
    flashOnboardLed(1, 50, 0);

    command.trim();

    if (command.startsWith("LED ")) {
        int params[4] = {0};
        int paramIndex = 0;
        int lastSpace = 3;
        
        while (paramIndex < 4 && lastSpace < command.length()) {
            int nextSpace = command.indexOf(' ', lastSpace + 1);
            if (nextSpace == -1) nextSpace = command.length();
            
            String param = command.substring(lastSpace + 1, nextSpace);
            params[paramIndex++] = param.toInt();
            lastSpace = nextSpace;
        }
        
        if (paramIndex == 4) {
            setLedColor(params[0], params[1], params[2], params[3]);
        } else {
                Serial.println("Invalid LED command format. Use: LED <index> <red> <green> <blue>");
        }
    } else if (command.startsWith("OFF")) {
        turnOffLeds();
    }
} 

void loop() {
    updateAndLogButtonStates();
    
    if (Serial.available() > 0) {
        String command = Serial.readStringUntil('\n');
        handleSerialCommand(command);
    }
}
